
def server_help():
    print('Поддерживаемые команды:\n'
          'users - список пользователей\n'
          'connected - список подключённых пользователей\n'
          'history - история входов пользователя\n'
          'exit - завершение работы сервера.\n'
          'help - справка')


def client_help():
    print('Поддерживаемые команды:\n'
          'message - отправить сообщение. Кому и текст будет запрошены отдельно.\n'
          'history - история сообщений\n'
          'contacts - список контактов\n'
          'edit - редактирование списка контактов\n'
          'help - вывести подсказки по командам\n'
          'exit - выход из программы')
